package edu.nyu.scps.puzzle;

import android.content.Context;
import android.graphics.Color;
import android.view.ViewPropertyAnimator;
import android.widget.RelativeLayout;


public class PuzzleView extends RelativeLayout {
    private int n = 3;
    private int emptyRow = n - 1;
    private int emptyCol = 0;
    private int tileWidth;
    private int tileHeight;
    private int margin = 1; //width of margin in pixels

    public PuzzleView(Context context) {
        super(context);
        setBackgroundColor(Color.WHITE);

        int[][] a = new int[][] {
                {R.drawable.i00, R.drawable.i01, R.drawable.i02}, //top row
                {R.drawable.i10, R.drawable.i11, R.drawable.i12}, //middle row
                {R.drawable.i20, R.drawable.i21, R.drawable.i22}  //bottom row
        };

        for (int row = 0; row < a.length; ++row) {
            for (int col = 0; col < a[row].length; ++col) {
                //Lower left tile is missing.
                if (row != emptyRow || col != emptyCol) {
                    TileView tileView = new TileView(context, row, col, a[row][col]);
                    addView(tileView);
                }
            }
        }
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);

        //Each .png file is 300 x 394 pixels.
        if (getWidth() <= getHeight() * 394 / 300) {
            //Use the full width of the screen.
            tileWidth = getWidth() / n;
            tileHeight = tileWidth * 394 / 300;
        } else {
            //Use the full height of the screen.
            tileHeight = getHeight() / n;
            tileWidth = tileHeight * 300 / 394;
        }

        for (int i = 0; i < getChildCount(); ++i) {
            TileView tileView = (TileView)getChildAt(i);
            int row = tileView.getRow();
            int col = tileView.getCol();
            tileView.layout(
                    colToPixel(col),
                    rowToPixel(row),
                    colToPixel(col) + tileWidth,
                    rowToPixel(row) + tileHeight
            );
        }
    }

    public void wasTouched(TileView tileView) {
        int horizontalDistance = Math.abs(tileView.getCol() - emptyCol); //in columns
        int verticalDistance   = Math.abs(tileView.getRow() - emptyRow); //in rows

        //If the TileView that was touched can move into the empty position,
        if (horizontalDistance + verticalDistance == 1) {

            //Move the TileView into the empty position.
            ViewPropertyAnimator viewPropertyAnimator = tileView.animate();
            viewPropertyAnimator.setDuration(200L);    //milliseconds
            float x = colToPixel(emptyCol);
            float y = rowToPixel(emptyRow);
            viewPropertyAnimator.x(x).y(y);

            //Swap the row and column of the TileView with the row and column of the empty position.
            int temp = tileView.getRow();
            tileView.setRow(emptyRow);
            emptyRow = temp;

            temp = tileView.getCol();
            tileView.setCol(emptyCol);
            emptyCol = temp;
        }
    }

    //Return the X coordinate of the left edge of a TileView in the given column.

    private int colToPixel(int col) {
        int centerOfPuzzleView = getWidth() / 2;
        int centerOfTileView = centerOfPuzzleView + (col - 1) * (tileWidth + margin);
        int leftEdgeOfTileView = centerOfTileView - tileWidth / 2;
        return leftEdgeOfTileView;
    }

    //Return the Y coordinate of the top edge of a TileView in the given row.

    private int rowToPixel(int row) {
        int centerOfPuzzleView = getHeight() / 2;
        int centerOfTileView = centerOfPuzzleView + (row - 1) * (tileHeight + margin);
        int topEdgeOfTileView = centerOfTileView - tileHeight / 2;
        return topEdgeOfTileView;
    }

}
